@@ -5,7 +5,7 @@ use base qw(LISM::Handler);
 use POSIX qw(strftime);
 use Config::IniFiles;
 use Encode;
-use Net::LDAP::Constant qw(:all);
+use LISM::Constant;
 use Data::Dumper;
 
 =head1 NAME
@@ -70,7 +70,7 @@ Check modify request.
 sub pre_modify
 {
     my $self = shift;
-    my ($dnp, $listp, $oldentryp) = @_;
+    my ($dnp, $listp, $oldentryp, $errorp) = @_;
     my $conf = $self->{_config};
 
     my $entryStr = "dn: ${$dnp}\n";
@@ -84,7 +84,9 @@ sub pre_modify
             push(@values, shift @list);
         }
 
-        if ($action eq 'DELETE') {
+        if ($action eq 'DELETE' && !@values) {
+            @values = ('');
+        } elsif ($action eq 'DELETE') {
             next;
         }
 
@@ -93,12 +95,12 @@ sub pre_modify
         }
     }
 
-    my $rc = $self->_checkValues($entryStr, 'modify', 'pre', $oldentryp ? ${$oldentryp} : '');
+    my $rc = $self->_checkValues($entryStr, 'modify', 'pre', $oldentryp ? ${$oldentryp} : '', $errorp);
 
     return $rc;
 }
 
-=head2 pre_add($dnp, $entryStrp)
+=head2 pre_add($dnp, $entryStrp, $oldentryp)
 
 Check add request.
 
@@ -107,10 +109,10 @@ Check add request.
 sub pre_add
 {
     my $self = shift;
-    my ($dnp, $entryStrp) = @_;
+    my ($dnp, $entryStrp, $oldentryp, $errorp) = @_;
     my $conf = $self->{_config};
 
-    my $rc = $self->_checkValues("dn: ${$dnp}\n${$entryStrp}[0]", 'add');
+    my $rc = $self->_checkValues("dn: ${$dnp}\n${$entryStrp}[0]", 'add', 'pre', $oldentryp, $errorp);
 
     return $rc;
 }
@@ -140,8 +142,14 @@ sub post_modify
         }
 
         if ($action eq 'DELETE') {
-            foreach my $value (@values) {
-                $entryStr .= "$attr: $value\n";
+            if (@values && $values[0] !~ /^ *$/) {
+                foreach my $value (@values) {
+                    $entryStr .= "$attr: $value\n";
+                }
+            } else {
+                foreach my $value ($oldentry =~ /^$attr: (.+)$/gmi) {
+                    $entryStr .= "$attr: $value\n";
+                }
             }
         } elsif ($action eq 'REPLACE') {
             foreach my $value ($oldentry =~ /^$attr: (.+)$/gmi) {
@@ -184,15 +192,27 @@ sub post_delete
     my ($dnp, $null, $oldentryp) = @_;
     my $conf = $self->{_config};
 
-    return $self->_checkValues("dn: ${$dnp}\n", 'delete', 'post', $oldentryp ? ${$oldentryp} : '');
+    return $self->_checkValues($oldentryp ? ${$oldentryp} : "dn: ${$dnp}\n", 'delete', 'post', $oldentryp ? ${$oldentryp} : '');
 }
 
 sub _checkValues
 {
     my $self = shift;
-    my ($entryStr, $func, $type, $oldentry) = @_;
+    my ($entryStr, $func, $type, $oldentry, $errorp) = @_;
     my $conf = $self->{_config};
     my $rc = LDAP_SUCCESS;
+    my $newentry = $entryStr;
+    if ($func ne 'add' && $oldentry) {
+        foreach my $attr (($oldentry =~ /^([^:]+):/gm)) {
+            if ($newentry !~ /^$attr:/mi) {
+                foreach my $value (($oldentry =~ /^$attr: (.+)$/gmi)) {
+                    $newentry .= "$attr: $value\n";
+                }
+            }
+        }
+    } elsif ($func eq 'search') {
+        $newentry = $entryStr;
+    }
 
     foreach my $rule (@{$conf->{check}}) {
         my ($dn) = ($entryStr =~ /^dn: (.*)\n/);
@@ -202,7 +222,7 @@ sub _checkValues
         if (defined($rule->{op}) && (','.$rule->{op}.',') !~ /,$func,/) {
             next;
         }
-        if (defined($rule->{filter}) && !LISM::Storage->parseFilter($rule->{filterobj}, encode('utf8', $entryStr))) {
+        if (defined($rule->{filter}) && !LISM::Storage->parseFilter($rule->{filterobj}, encode('utf8', $newentry))) {
             next;
         }
         if (defined($rule->{entry})) {
@@ -226,7 +246,7 @@ sub _checkValues
             }
         }
         my %params;
-        if ($type ne 'post') {
+        if ($type ne 'post' || $func eq 'search') {
             if (defined($rule->{param})) {
                 foreach my $param (keys(%{$rule->{param}})) {
                    my $value = $self->_getParam($rule->{param}{$param}, $dn, $param);
@@ -238,37 +258,57 @@ sub _checkValues
         }
         foreach my $attr (keys %{$rule->{attr}}) {
             my $cattr = $rule->{attr}{$attr};
-            my @values = ($entryStr =~ /^$attr: (.+)$/gmi);
+            my @tmpvals = ($entryStr =~ /^$attr: (.+)$/gmi);
+            my @values;
+            foreach my $value (@tmpvals) {
+                if (defined($cattr->{notrule})) {
+                    my $notrule = $cattr->{notrule}[0];
+                    if ($value =~ /$notrule/i) {
+                        next;
+                    }
+                }
+                push(@values, $value);
+            }
             if (defined($cattr->{maxentries})) {
                 my $opts = $cattr->{maxentries}[0];
                 my $mrc;
                 if ($type eq 'post') {
                     my @delvals;
+                    my @tmpvals;
                     if ($func eq 'delete') {
-                        @delvals = ($oldentry =~ /^$attr: (.+)$/gmi);
+                        @tmpvals = ($oldentry =~ /^$attr: ([^ ]+)$/gmi);
                     } else {
-                        @delvals = ($entryStr =~ /^$attr: (.+)$/gmi);
+                        @tmpvals = ($entryStr =~ /^$attr: ([^ ]+)$/gmi);
                     }
-                    ($mrc) = !$self->_updateCurrentEntries($opts, $dn, $func, $attr, $oldentry, @delvals);
+                    if (defined($cattr->{notrule})) {
+                        my $notrule = $cattr->{notrule}[0];
+                        foreach my $value (@tmpvals) {
+                            if ($value =~ /$notrule/i) {
+                                next;
+                            }
+                            push(@delvals, $value);
+                        }
+                    } else {
+                        @delvals = @tmpvals;
+                    }
+                    @delvals = $self->_unique(@delvals);
+                    ($mrc) = !$self->_updateCurrentEntries($opts, $dn, $func, $attr, @delvals);
                     if (!defined($mrc)) {
                         return LDAP_USER_CANCELED;
                     }
                 } elsif ($func eq 'add' || $func eq 'modify') {
-                    my $mentry;
                     my @addvals;
                     if ($func eq 'modify') {
-                        $mentry = $oldentry;
                         foreach my $value (@values) {
                             if ($oldentry !~ /^$attr: $value$/mi) {
                                 push(@addvals, $value);
                             }
                         }
                     } else {
-                        $mentry = $entryStr;
-                        @addvals = ($entryStr =~ /^$attr: (.+)$/gmi);
+                        @addvals = @values;
                     }
                     my $error;
-                    ($mrc, $error) = $self->_checkMaxEntries($opts, $dn, $attr, $mentry, @addvals);
+                    ($mrc, $error) = $self->_checkMaxEntries($opts, $dn, $attr, @addvals);
                     if (!defined($mrc)) {
                         return LDAP_OTHER;
                     } elsif (!$mrc) {
@@ -280,8 +320,9 @@ sub _checkValues
             if ($type eq 'post') {
                 next;
             }
-            if ($func eq 'add' && defined($cattr->{required}) && $cattr->{required}[0] eq 'on' && !@values) {
+            if (defined($cattr->{required}) && $cattr->{required}[0] eq 'on' && ((($func eq 'add' || $func eq 'search') && (!@values || $values[0] =~ /^ *$/)) || ($func eq 'modify' && $entryStr =~ /$attr: / && (!@values || $values[0] =~ /^ *$/)))) {
                 $self->_perror("$attr in $dn is required value");
+                ${$errorp} = "$attr is required value" if ref($errorp);
                 $rc = LDAP_CONSTRAINT_VIOLATION;
             }
             if (defined($cattr->{valexists}) && @values && $values[0] !~ /^ *$/) {
@@ -292,14 +333,18 @@ sub _checkValues
                     my ($filterval) = ($dn =~ /($regexp)/i);
                     $filter = "(dn=$filterval)";
                 }
+                my ($base) = ($dn =~ /($opts->{base})$/i);
 
-                my @vals = $self->_searchLism($opts, $filter);
+                my @vals = $self->_searchLism($opts, $filter, $base);
                 if (defined($opts->{option}) && $opts->{option} =~ /addval=([^&]+)/) {
                     push(@vals, split(/,/, $1));
                 }
                 foreach my $value (@values) {
-                    if (!grep(/^$value$/, @vals)) {
+                    my $regex_val = $value;
+                    $regex_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
+                    if (!grep(/^$regex_val$/, @vals)) {
                         $self->_perror("$attr=$value in $dn is invalid: value doesn't exist in entry");
+                        ${$errorp} = "$attr=$value is invalid: value doesn't exist in entry" if ref($errorp);
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                     }
                 }
@@ -310,10 +355,12 @@ sub _checkValues
                 }
                 if (defined($cattr->{minlen}) && length($value) < $cattr->{minlen}[0]) {
                     $self->_perror("$attr=$value in $dn is too short");
+                    ${$errorp} = "$attr=$value is too short" if ref($errorp);
                     $rc = LDAP_CONSTRAINT_VIOLATION;
                 }
                 if (defined($cattr->{maxlen}) && length($value) > $cattr->{maxlen}[0]) {
                     $self->_perror("$attr=$value in $dn is too long");
+                    ${$errorp} = "$attr=$value is too long" if ref($errorp);
                     $rc = LDAP_CONSTRAINT_VIOLATION;
                 }
                 if (defined($cattr->{regexp})) {
@@ -321,6 +368,7 @@ sub _checkValues
                     $regexp = $self->_replaceParam($regexp, %params);
                     if ($value !~ /$regexp/i) {
                         $self->_perror("$attr=$value in $dn is invalid: regular expression is $regexp");
+                        ${$errorp} = "$attr=$value is invalid: regular expression is $regexp" if ref($errorp);
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                     }
                 }
@@ -329,32 +377,80 @@ sub _checkValues
                     eval "\$ecode = $cattr->{function}[0](\$value)";
                     if (!$ecode) {
                         $self->_perror("$attr=$value in $dn is invalid: function is $cattr->{function}[0]($ecode)");
+                        ${$errorp} = "$attr=$value is invalid" if ref($errorp);
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                     }
                 }
+                if (defined($cattr->{entryunique})) {
+                    my $tmpval = $value;
+                    $tmpval =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
+                    foreach my $uattr (split(/, */, $cattr->{entryunique}[0])) {
+                        my $match = 0;
+                        if ($entryStr =~ /^$uattr: /mi) {
+                            if ($entryStr =~ /^$uattr: $tmpval$/mi) {
+                                $match = 1;
+                            }
+                        } elsif ($oldentry && $oldentry =~ /^$uattr: /mi) {
+                            if ($oldentry =~ /^$uattr: $tmpval$/mi) {
+                                $match = 1;
+                            }
+                        }
+                        if ($match) {
+                            $self->_perror("$attr=$value in $dn is invalid: value exist in $uattr");
+                            ${$errorp} = "$attr=$value is invalid: value exist in $uattr" if ref($errorp);
+                            $rc = LDAP_CONSTRAINT_VIOLATION;
+                            last;
+                        }
+                    }
+                }
                 if (defined($cattr->{lismexist})) {
                     my $opts = $cattr->{lismopts};
-                    my $filter = "($opts->{attr}=$value)";
-                    if (defined($opts->{filter})) {
-                        $filter = "(&$filter$opts->{filter})";
+                    my $filter;
+                    my $base;
+                    if ($opts->{attr} eq 'path') {
+                        ($base) = ($dn =~ /($opts->{base})$/i);
+                        $base = LISM::Handler::_path2dn($value, 'ou', 1).',ou=Organizations,'.$base;
+                    } elsif ($opts->{attr} eq 'dn' && defined($opts->{filter})) {
+                        $base = $value;
+                    } else {
+                        my $tmpval = $value;
+                        $tmpval =~ s/(?<!\\)\\/\\\\/g;
+                        $tmpval =~ s/([\(\)*])/\\$1/g;
+                        $filter = "($opts->{attr}=$tmpval)";
+                        ($base) = ($dn =~ /($opts->{base})$/i);
                     }
-                    my @vals = $self->_searchLism($opts, $filter);
+                    if (defined($opts->{filter})) {
+                        $filter = $filter ? "(&$filter$opts->{filter})" : $opts->{filter};
+                    }
+                    my @vals = $self->_searchLism($opts, $filter, $base);
                     if (!@vals || !$vals[0]) {
                         $self->_perror("$attr=$value in $dn is invalid: value doesn't exist in data");
+                        ${$errorp} = "$attr=$value is invalid: value doesn't exist in data" if ref($errorp);
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                     }
                 }
                 if (defined($cattr->{lismunique})) {
                     my $opts = $cattr->{lismopts};
-                    my $filter = "($opts->{attr}=$value)";
+                    my $tmpval = $value;
+                    $tmpval =~ s/(?<!\\)\\/\\\\/g;
+                    $tmpval =~ s/([\(\)*])/\\$1/g;
+                    my $filter = "($opts->{attr}=$tmpval)";
+                    if (defined($opts->{attrs})) {
+                        foreach my $luattr (@{$opts->{attrs}}) {
+                            $filter = "(|$filter($luattr=$tmpval))";
+                        }
+                    }
                     if (defined($opts->{filter})) {
                         $filter = "(&$filter$opts->{filter})";
                     }
                     my ($id) = ($dn =~ /^[^=]+=([^,]+),/);
                     $filter =~ s/\%i/$id/g;
-                    my @vals = $self->_searchLism($opts, $filter);
+                    $filter =~ s/\%a/$tmpval/g;
+                    my ($base) = ($dn =~ /($opts->{base})$/i);
+                    my @vals = $self->_searchLism($opts, $filter, $base);
                     if (@vals && $vals[0]) {
                         $self->_perror("$attr=$value in $dn is invalid: value already exist in data");
+                        ${$errorp} = "$attr=$value is invalid: value already exist in data" if ref($errorp);
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                     }
                 }
@@ -364,6 +460,7 @@ sub _checkValues
                         return LDAP_OTHER;
                     } elsif (!$prc) { 
                         $self->_perror("$attr=$value in $dn is invalid: $error");
+                        ${$errorp} = "$attr=$value is invalid: $error" if ref($errorp);
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                     }
                 }
@@ -393,14 +490,27 @@ sub _getParam
     my $self = shift;
     my ($opts, $dn, $param) = @_;
 
-    my ($base) = ($dn =~ /($opts->{base})/i);
+    my $expire = defined($opts->{expire}) ? $opts->{expire} : 300;
+    my $base;
+    if (defined($opts->{suffix})) {
+        $base = $opts->{suffix};
+    } else {
+        ($base) = ($dn =~ /($opts->{base})/i);
+    }
     if (!$base) {
         return 1;
     }
 
+    if (!defined($self->{lism}->{check_param})) {
+        $self->{lism}->{check_param} = {};
+    }
+
     my %params;
-    if (defined($self->{lism}->{bind}{check_param}{$base})) {
-        %params = %{$self->{lism}->{bind}{check_param}{$base}};
+    if (defined($self->{lism}->{check_param}{$base})) {
+        %params = %{$self->{lism}->{check_param}{$base}};
+        if (defined($params{$param}) && ${$params{$param}}{timestamp} + $expire < time()) {
+            undef($params{$param});
+        }
     }
     if (!%params || !defined($params{$param})) {
         my $filter = defined($opts->{filter}) ? $opts->{filter} : '(objectClass=*)';
@@ -410,32 +520,36 @@ sub _getParam
             return undef;
         }
 
+        $params{$param} = {timestamp => time()};
         if (@entries) {
             my @values;
-            foreach my $attr (split(/, */, $opts->{attr})) {
-                my @tmpvals = ($entries[0] =~ /^$attr: (.*)$/gmi);
-                if (@tmpvals) {
-                    push(@values, @tmpvals);
+            for (my $i = 0; $i < @entries; $i++) {
+                foreach my $attr (split(/, */, $opts->{attr})) {
+                    my @tmpvals = ($entries[$i] =~ /^$attr: (.*)$/gmi);
+                    if (@tmpvals) {
+                        foreach my $tmpval (@tmpvals) {
+                            if ($tmpval !~ /^ *$/ && !grep(/$tmpval$/i, @values)) {
+                                push(@values, $tmpval);
+                            }
+                        }
+                    }
                 }
             }
             if (@values) {
                 if (@values > 1) {
-                    $params{$param} = '('.join('|', @values).')';
+                    ${$params{$param}}{value} = '('.join('|', @values).')';
                 } else {
-                    $params{$param} = $values[0];
+                    ${$params{$param}}{value} = $values[0];
                 }
             }
         }
-        if (!defined($params{$param})) {
-            $params{$param} = defined($opts->{default}) ? $opts->{default} : '';
+        if (!defined(${$params{$param}}{value})) {
+            ${$params{$param}}{value} = defined($opts->{default}) ? $opts->{default} : '';
         }
-        if (!defined($self->{lism}->{bind}{check_param})) {
-            $self->{lism}->{bind}{check_param} = {};
-        }
-        $self->{lism}->{bind}{check_param}{$base} = \%params;
+        $self->{lism}->{check_param}{$base} = \%params;
     }
 
-    return $params{$param};
+    return ${$params{$param}}{value};
 }
 
 sub _checkPwdPolicy
@@ -545,7 +659,7 @@ sub _checkPwdPolicy
 sub _checkMaxEntries
 {
     my $self = shift;
-    my ($opts, $dn, $attr, $entryStr, @values) = @_;
+    my ($opts, $dn, $attr, @values) = @_;
     my $conf = $self->{_config};
 
     my ($base) = ($dn =~ /($opts->{dn})/i);
@@ -562,8 +676,6 @@ sub _checkMaxEntries
     if (@entries) {
         my $checkEntry = $entries[0];
         if ($attr) {
-            my $idattr = defined($opts->{uid}) ? $opts->{uid} : $opts->{service};
-            my @ids = ($entryStr =~ /^$idattr: (.+)$/gmi);
             undef($opts->{increment});
             $opts->{increment} = {};
             foreach my $value (@values) {
@@ -571,42 +683,22 @@ sub _checkMaxEntries
                     next;
                 }
 
-                foreach my $id (@ids) {
-                    my $maxattr = $opts->{max};
-                    my $currentattr = $opts->{current};
-                    $maxattr =~ s/\%a/$value/;
-                    $maxattr =~ s/\%i/$id/;
-                    $currentattr =~ s/\%a/$value/;
-                    $currentattr =~ s/\%i/$id/;
-                    my ($max) = ($checkEntry =~ /^$maxattr: (.*)$/mi);
-                    my ($current) = ($checkEntry =~ /^$currentattr: (.*)$/mi);
-                    if (!defined($max)) {
-                        return 1;
-                    }
-                    my $service;
-                    if (defined($opts->{uid})) {
-                        $service = $value;
-                    } else {
-                        $service = $id;
-                    }
-
-                    my $filterStr = $opts->{check};
-                    $filterStr =~ s/\%a/$value/g;
-                    $filterStr =~ s/\%i/$id/g;
-                    my ($idrc, @identries) = $self->{lism}->search($base, 2, 0, 0, 0, $filterStr, 0, $idattr);
-                    if ($idrc) {
-                        $self->log(level => 'err', message => "searching entries with $id failed($idrc)");
-                        return undef;
-                    }
-                    if (!@identries) {
-                        if (!defined($opts->{increment}->{$service})) {
-                            $opts->{increment}->{$service} = 0;
-                        }
-                        $opts->{increment}->{$service}++;
-                        if ($current + $opts->{increment}->{$service} > $max) {
-                            return (0, "$maxattr=$max");
-                        }
-                    }
+                my $maxattr = $opts->{max};
+                my $currentattr = $opts->{current};
+                $maxattr =~ s/\%a/$value/;
+                $currentattr =~ s/\%a/$value/;
+                my ($max) = ($checkEntry =~ /^$maxattr: (.*)$/mi);
+                my ($current) = ($checkEntry =~ /^$currentattr: (.*)$/mi);
+                if (!defined($max)) {
+                    return (0, "$maxattr=0");
+                }
+                my $service = $value;
+                if (!defined($opts->{increment}->{$service})) {
+                    $opts->{increment}->{$service} = 0;
+                }
+                $opts->{increment}->{$service}++;
+                if ($current + $opts->{increment}->{$service} > $max) {
+                    return (0, "$maxattr=$max");
                 }
             }
         } else {
@@ -627,7 +719,7 @@ sub _checkMaxEntries
 sub _updateCurrentEntries
 {
     my $self = shift;
-    my ($opts, $dn, $func, $attr, $entryStr, @delvals) = @_;
+    my ($opts, $dn, $func, $attr, @delvals) = @_;
     my $conf = $self->{_config};
 
     my ($base) = ($dn =~ /($opts->{dn})/i);
@@ -635,78 +727,79 @@ sub _updateCurrentEntries
         return 1;
     }
 
-    my ($rc, @entries) = $self->{lism}->search($base, 0, 0, 0, 0, '(objectClass=*)', 0);
-    if ($rc) {
-        $self->log(level => 'err', message => "searching max number($base) failed($rc)");
-        return undef;
-    }
+    for (my $i = 0; $i < 5; $i++) {
+        my ($rc, @entries) = $self->{lism}->search($base, 0, 0, 0, 0, '(objectClass=*)', 0);
+        if ($rc) {
+            $self->log(level => 'err', message => "searching max number($base) failed($rc)");
+            return undef;
+        }
 
-    if (@entries) {
-        my @list;
-        my $checkEntry = $entries[0];
-        if ($attr) {
-            my %currentvals;
-            if (($func eq 'add' || $func eq 'modify') && defined($opts->{increment})) {
-                foreach my $service (keys %{$opts->{increment}}) {
-                    my $currentattr = $opts->{current};
-                    $currentattr =~ s/\%[ai]/$service/;
-                    my ($current) = ($checkEntry =~ /^$currentattr: (.*)$/mi);
-                    $currentvals{$service} = $current + ${$opts->{increment}}{$service};
+        if (@entries) {
+            my @list;
+            my $checkEntry = $entries[0];
+            if ($attr) {
+                my %currentvals;
+                my %oldvals;
+                if (($func eq 'add' || $func eq 'modify') && defined($opts->{increment})) {
+                    foreach my $service (keys %{$opts->{increment}}) {
+                        my $currentattr = $opts->{current};
+                        $currentattr =~ s/\%[ai]/$service/;
+                        my ($current) = ($checkEntry =~ /^$currentattr: (.*)$/mi);
+                        if (defined($current)) {
+                            $oldvals{$service} = $current;
+                        }
+                        $currentvals{$service} = (defined($current) ? $current : 0) + ${$opts->{increment}}{$service};
+                    }
                 }
-            }
-            if (($func eq 'modify' || $func eq 'delete') && @delvals && $delvals[0] !~ /^ *$/) {
-                my $idattr = defined($opts->{uid}) ? $opts->{uid} : $opts->{service};
-                my @ids = ($entryStr =~ /^$idattr: (.+)$/gmi);
-                foreach my $value (@delvals) {
-                    foreach my $id (@ids) {
-                        my $service;
-                        if (defined($opts->{uid})) {
-                            $service = $value;
+                if (($func eq 'modify' || $func eq 'delete') && @delvals && $delvals[0] !~ /^ *$/) {
+                    foreach my $value (@delvals) {
+                        my $service = $value;
+                        if (defined($currentvals{$service})) {
+                            $currentvals{$service}--;
                         } else {
-                            $service = $id;
-                        }
-                        my $filterStr = $opts->{check};
-                        $filterStr =~ s/\%a/$value/g;
-                        $filterStr =~ s/\%i/$id/g;
-                        my ($idrc, @identries) = $self->{lism}->search($base, 2, 0, 0, 0, $filterStr, 0, $idattr);
-                        if ($idrc) {
-                            $self->log(level => 'err', message => "searching entries with $id failed($idrc)");
-                            return undef;
-                        }
-                        if (!@identries) {
-                            if (defined($currentvals{$service})) {
-                                $currentvals{$service}--;
-                            } else {
-                                my $currentattr = $opts->{current};
-                                $currentattr =~ s/\%a/$value/;
-                                $currentattr =~ s/\%i/$id/;
-                                my ($current) = ($checkEntry =~ /^$currentattr: (.*)$/mi);
-                                $currentvals{$service} = $current - 1;
+                            my $currentattr = $opts->{current};
+                            $currentattr =~ s/\%a/$value/;
+                            my ($current) = ($checkEntry =~ /^$currentattr: (.*)$/mi);
+                            if (defined($current)) {
+                                $oldvals{$service} = $current;
                             }
+                            $currentvals{$service} = (defined($current) ? $current : 0) - 1;
                         }
                     }
                 }
-            }
-            foreach my $service (keys %currentvals) {
-                my $currentattr = $opts->{current};
-                $currentattr =~ s/\%[ai]/$service/;
-                push(@list, 'REPLACE', $currentattr, $currentvals{$service});
-            }
-        } else {
-            my ($current) = ($checkEntry =~ /^$opts->{current}: (.*)$/mi);
-            if ($func eq 'add') {
-                $current++;
+                foreach my $service (keys %currentvals) {
+                    my $currentattr = $opts->{current};
+                    $currentattr =~ s/\%a/$service/;
+                    if (defined($oldvals{$service})) {
+                        push(@list, 'DELETE', $currentattr, $oldvals{$service});
+                    }
+                    push(@list, 'ADD', $currentattr, $currentvals{$service});
+                }
             } else {
-                $current--;
+                my ($current) = ($checkEntry =~ /^$opts->{current}: (.*)$/mi);
+                my $oldval = $current;
+                $current = defined($current) ? $current : 0;
+                if ($func eq 'add') {
+                    $current++;
+                } else {
+                    $current--;
+                }
+                if (defined($oldval)) {
+                    push(@list, 'DELETE', $opts->{current}, $oldval);
+                }
+                push(@list, 'ADD', $opts->{current}, $current);
             }
-            @list = ('REPLACE', $opts->{current}, $current);
-        }
 
-        if (@list) {
-            $rc = $self->{lism}->modify($base, @list);
-            if ($rc) {
-                $self->log(level => 'err', message => "modifying current number($base) failed($rc)");
-                return undef;
+            if (@list) {
+                $rc = $self->{lism}->modify($base, @list);
+                if ($rc && $rc != LDAP_NO_SUCH_ATTRIBUTE && $rc != LDAP_TYPE_OR_VALUE_EXISTS) {
+                    $self->log(level => 'err', message => "modifying current number($base) failed($rc)");
+                    return undef;
+                } if (!$rc) {
+                    return 1;
+                }
+            } else {
+                last;
             }
         }
     }
@@ -771,6 +864,7 @@ sub _checkConfig
     if (defined($conf->{check})) {
         foreach my $rule (@{$conf->{check}}) {
             if (defined($rule->{filter})) {
+                $rule->{filter} =~ s/&amp;/&/g;
                 $rule->{filterobj} = Net::LDAP::Filter->new($rule->{filter});
             }
             if (defined($rule->{entry}) && defined($rule->{entry}[0]->{maxentries})) {
@@ -793,9 +887,13 @@ sub _checkConfig
                         }
                     }
                     my $lismopts = {};
-                    my ($base, $attr, $scope, $filter, $option) = split(/\?/, $rule->{attr}{$cattr}->{$name}[0]);
+                    my ($base, $attrsStr, $scope, $filter, $option) = split(/\?/, $rule->{attr}{$cattr}->{$name}[0]);
                     $lismopts->{base} = $base;
+                    my ($attr, @attrs) = split(/,/, $attrsStr);
                     $lismopts->{attr} = $attr;
+                    if (@attrs) {
+                        $lismopts->{attrs} = \@attrs;
+                    }
                     if ($scope) {
                         $lismopts->{scope} = $scope;
                     }
@@ -808,15 +906,10 @@ sub _checkConfig
                     $rule->{attr}{$cattr}->{lismopts} = $lismopts;
                 } elsif (defined($rule->{attr}{$cattr}->{maxentries})) {
                     my $maxentries = $rule->{attr}{$cattr}->{maxentries}[0];
-                    if (!defined($maxentries->{dn}) || !defined($maxentries->{max}) || !defined($maxentries->{current}) || !defined($maxentries->{check})) {
+                    if (!defined($maxentries->{dn}) || !defined($maxentries->{max}) || !defined($maxentries->{current})) {
                         $self->log(level => 'alert', message => "Set dn,max,current,check in maxentries");
                         return 1;
                     }
-                    $maxentries->{check} =~ s/&amp;/&/g;
-                    if (!defined($maxentries->{uid}) && !defined($maxentries->{service})) {
-                        $self->log(level => 'alert', message => "Set id or service in maxentries");
-                        return 1;
-                    }
                 }
             }
         }
