@@ -2,7 +2,7 @@ package LISM::Handler::Rewrite;
 
 use strict;
 use base qw(LISM::Handler);
-use Net::LDAP::Constant qw(:all);
+use LISM::Constant;
 use Net::LDAP::Filter;
 use Encode;
 use LISM::Storage;
@@ -183,27 +183,25 @@ sub post_search
                 my $substitution = $rule->{substitution};
                 $substitution = $self->_rewritePattern($substitution, '%0', $entryStr);
 
-                my $str = $self->_rewriteParse($rule->{match}, $substitution, $dn, \%rwcache);
+                my $str = $self->_rewriteParse($rule->{match}, $substitution, $dn, \%rwcache, 1);
                 if (!$str) {
-                    $self->log(level => 'err', message => "search result rewrite \"$dn\" failed");
+                    $self->log(level => 'err', message => "search result rewrite rule \"$rule->{substitution}\" to \"$dn\" failed");
                     next;
                 }
                 $line[0] = "dn: $str";
 
                 my @replaced;
                 for (my $j = 1; $j < @line; $j++) {
+                    my $org = $line[$j];
                     $line[$j] = $self->_rewriteParse($rule->{match}, $substitution, $line[$j], \%rwcache);
                     if (!defined($line[$j])) {
-                        $self->log(level => 'err', message => "search result rewrite \"$dn\" failed");
-                        $rc = -1;
-                        last;
+                        $self->log(level => 'err', message => "search result rewrite rule \"$rule->{substitution}\" to \"$dn\" failed");
+                        push(@replaced, $org);
                     } elsif ($line[$j]) {
                         push(@replaced, split(/\n/, $line[$j]));
                     }
                 }
-                if (!$rc) {
-                    ${$entriesp}[$i] = "$line[0]\n".join("\n", $self->_unique(@replaced))."\n";
-                }
+                ${$entriesp}[$i] = "$line[0]\n".join("\n", $self->_unique(@replaced))."\n";
             }
         }
     }
@@ -222,7 +220,7 @@ Rewrite dn and attributes, values before modify operation is done.
 sub pre_modify
 {
     my $self = shift;
-    my ($dnp, $listp) = @_;
+    my ($dnp, $listp, $oldentryp) = @_;
     my $conf = $self->{_config};
 
     foreach my $rule (@{$conf->{rewrite}}) {
@@ -235,10 +233,93 @@ sub pre_modify
             my $substitution = $rule->{substitution};
             my $modlist = "${$dnp}\n".join("\n", @{$listp});
             $substitution = $self->_rewritePattern($substitution, '%0', $modlist);
+            if ($oldentryp) {
+                $substitution = $self->_rewritePattern($substitution, '%-', ${$oldentryp});
+            }
+
+            my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache, 1);
+            if (!$str) {
+                $self->log(level => 'err', message => "modify rewrite rule \"$rule->{substitution}\" to \"${$dnp}\" failed");
+                return LDAP_OPERATIONS_ERROR;
+            }
+            (${$dnp}) = split(/\n/, $str);
+
+            my @mod_list;
+            while (@{$listp} > 0) {
+                my $action = shift @{$listp};
+                my $attr = shift @{$listp};
+                my @values;
+                my %replaced;
+
+                $str = $self->_rewriteParse($rule->{match}, $substitution, "$action: $attr");
+                ($attr) = ($str =~ /^[^:]+: (.*)$/);
+                while (@{$listp} > 0 && ${$listp}[0] !~ /^(ADD|DELETE|REPLACE)$/) {
+                    push(@values, shift @{$listp});
+                }
+
+                if (defined($rule->{modop}) && $rule->{modop} !~ /$action/i) {
+                    push(@mod_list, ($action, $attr, @values));
+                } elsif (@values) {
+                    my %rwactions;
+                    for (my $i =0; $i < @values; $i++) {
+                        $str = $self->_rewriteParse($rule->{match}, $substitution, "$attr: ".$values[$i], \%rwcache);
+                        if (!defined($str)) {
+                            $self->log(level => 'err', message => "modify rewrite rule \"$rule->{substitution}\" to \"$attr: $values[$i]\" in \"${$dnp}\" failed");
+                            return LDAP_OPERATIONS_ERROR;
+                        } elsif ($str) {
+                            foreach my $line (split(/\n/, $str)) {
+                                if ($line =~ /^ADD|DELETE|REPLACE/) {
+                                    my ($rwaction, $rwattr) = ($line =~ /(^[^:]*): (.*)$/);
+                                    if (!defined($replaced{$rwattr})) {
+                                        $rwactions{$rwattr} = $rwaction;
+                                    }
+                                    next;
+                                }
+                                my ($rwattr, $value) = ($line =~ /(^[^:]*): (.*)$/);
+                                if (!defined($replaced{$rwattr})) {
+                                    @{$replaced{$rwattr}} = ();
+                                }
+                                push(@{$replaced{$rwattr}}, $value);
+                            }
+                        }
+                    }
+                    foreach my $rwattr (keys %replaced) {
+                        push(@mod_list, (defined($rwactions{$rwattr}) ? $rwactions{$rwattr} : $action, $rwattr, $self->_unique(@{$replaced{$rwattr}})));
+                    }
+                } else {
+                    push(@mod_list, ($action, $attr));
+                }
+            }
+            @{$listp} = @mod_list;
+        }
+    }
+
+    return LDAP_SUCCESS;
+}
+
+sub post_modify
+{
+    my $self = shift;
+    my ($dnp, $listp, $oldentryp) = @_;
+    my $conf = $self->{_config};
+
+    foreach my $rule (@{$conf->{rewrite}}) {
+        if ($rule->{context} eq 'modifyResult') {
+            if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
+                next;
+            }
+
+            my %rwcache;
+            my $substitution = $rule->{substitution};
+            my $modlist = "${$dnp}\n".join("\n", @{$listp});
+            $substitution = $self->_rewritePattern($substitution, '%0', $modlist);
+            if ($oldentryp) {
+                $substitution = $self->_rewritePattern($substitution, '%-', ${$oldentryp});
+            }
 
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
-                $self->log(level => 'err', message => "modify rewrite \"${$dnp}\" failed");
+                $self->log(level => 'err', message => "modify result rewrite rule \"$rule->{substitution}\" to \"${$dnp}\" failed");
                 return LDAP_OPERATIONS_ERROR;
             }
             (${$dnp}) = split(/\n/, $str);
@@ -259,13 +340,21 @@ sub pre_modify
                 if (defined($rule->{modop}) && $rule->{modop} !~ /$action/i) {
                     push(@mod_list, ($action, $attr, @values));
                 } elsif (@values) {
+                    my %rwactions;
                     for (my $i =0; $i < @values; $i++) {
                         $str = $self->_rewriteParse($rule->{match}, $substitution, "$attr: ".$values[$i], \%rwcache);
                         if (!defined($str)) {
-                            $self->log(level => 'err', message => "modify rewrite \"${$dnp}\" failed");
+                            $self->log(level => 'err', message => "modify result rewrite rule \"$rule->{substitution}\" to \"$attr: $values[$i]\" in \"${$dnp}\" failed");
                             return LDAP_OPERATIONS_ERROR;
                         } elsif ($str) {
                             foreach my $line (split(/\n/, $str)) {
+                                if ($line =~ /^ADD|DELETE|REPLACE/) {
+                                    my ($rwaction, $rwattr) = ($line =~ /(^[^:]*): (.*)$/);
+                                    if (!defined($replaced{$rwattr})) {
+                                        $rwactions{$rwattr} = $rwaction;
+                                    }
+                                    next;
+                                }
                                 my ($rwattr, $value) = ($line =~ /(^[^:]*): (.*)$/);
                                 if (!defined($replaced{$rwattr})) {
                                     @{$replaced{$rwattr}} = ();
@@ -275,7 +364,7 @@ sub pre_modify
                         }
                     }
                     foreach my $rwattr (keys %replaced) {
-                        push(@mod_list, ($action, $rwattr, $self->_unique(@{$replaced{$rwattr}})));
+                        push(@mod_list, (defined($rwactions{$rwattr}) ? $rwactions{$rwattr} : $action, $rwattr, $self->_unique(@{$replaced{$rwattr}})));
                     }
                 } else {
                     push(@mod_list, ($action, $attr));
@@ -315,9 +404,9 @@ sub pre_add
             my $substitution = $rule->{substitution};
             $substitution = $self->_rewritePattern($substitution, '%0', "${$dnp}\n${$entryStrp}[0]");
 
-            my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
+            my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache, 1);
             if (!$str) {
-                $self->log(level => 'err', message => "add rewrite \"${$dnp}\" failed");
+                $self->log(level => 'err', message => "add rewrite rule \"$rule->{substitution}\" to \"${$dnp}\" failed");
                 return LDAP_OPERATIONS_ERROR;
             }
             (${$dnp}) = split(/\n/, $str);
@@ -327,7 +416,49 @@ sub pre_add
             for (my $i = 0; $i < @line; $i++) {
                 $line[$i] = $self->_rewriteParse($rule->{match}, $substitution, $line[$i], \%rwcache);
                 if (!defined($line[$i])) {
-                    $self->log(level => 'err', message => "add rewrite \"${$dnp}\" failed");
+                    $self->log(level => 'err', message => "add rewrite rule \"$rule->{substitution}\" to \"$line[$i]\" in \"${$dnp}\" failed");
+                    return LDAP_OPERATIONS_ERROR;
+                }
+            }
+            ${$entryStrp}[0] = join("\n", $self->_unique(@line))."\n";
+        }
+    }
+
+    return LDAP_SUCCESS;
+}
+
+sub post_add
+{
+    my $self = shift;
+    my ($dnp, $entryStrp) = @_;
+    my $conf = $self->{_config};
+
+    foreach my $rule (@{$conf->{rewrite}}) {
+        if ($rule->{context} eq 'addResult') {
+            if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
+                next;
+            }
+            if (defined($rule->{filter}) && !LISM::Storage->parseFilter($rule->{filterobj}, "${$dnp}\n${$entryStrp}[0]")) {
+                next;
+            }
+
+            my %rwcache;
+            my $substitution = $rule->{substitution};
+            $substitution = $self->_rewritePattern($substitution, '%0', "${$dnp}\n${$entryStrp}[0]");
+
+            my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
+            if (!$str) {
+                $self->log(level => 'err', message => "add result rewrite rule \"$rule->{substitution}\" to \"${$dnp}\" failed");
+                return LDAP_OPERATIONS_ERROR;
+            }
+            (${$dnp}) = split(/\n/, $str);
+
+            my (@line) = split(/\n/, ${$entryStrp}[0]);
+
+            for (my $i = 0; $i < @line; $i++) {
+                $line[$i] = $self->_rewriteParse($rule->{match}, $substitution, $line[$i], \%rwcache);
+                if (!defined($line[$i])) {
+                    $self->log(level => 'err', message => "add result rewrite rule \"$rule->{substitution}\" to \"$line[$i]\" in \"${$dnp}\" failed");
                     return LDAP_OPERATIONS_ERROR;
                 }
             }
@@ -362,14 +493,14 @@ sub pre_modrdn
 
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
-                $self->log(level => 'err', message => "modrdn rewrite \"${$dnp}\" failed");
+                $self->log(level => 'err', message => "modrdn rewrite rule \"$rule->{substitution}\" to \"${$dnp}\" failed");
                 return LDAP_OPERATIONS_ERROR;
             }
             (${$dnp}) = split(/\n/, $str);
 
             $str = $self->_rewriteParse($rule->{match}, $substitution, ${$argsp}[0], \%rwcache);
             if (!$str) {
-                $self->log(level => 'err', message => "modrdn rewrite \"${$dnp}\" failed");
+                $self->log(level => 'err', message => "modrdn rewrite rule \"$rule->{substitution}\" to \"${$argsp}[0]\" in \"${$dnp}\" failed");
                 return LDAP_OPERATIONS_ERROR;
             }
             ${$argsp}[0] = $str;
@@ -390,7 +521,7 @@ Rewrite dn before delete operation is done.
 sub pre_delete
 {
     my $self = shift;
-    my ($dnp) = @_;
+    my ($dnp, $argsp, $oldentryp) = @_;
     my $conf = $self->{_config};
 
     foreach my $rule (@{$conf->{rewrite}}) {
@@ -402,10 +533,44 @@ sub pre_delete
             my %rwcache;
             my $substitution = $rule->{substitution};
             $substitution = $self->_rewritePattern($substitution, '%0', ${$dnp});
+            if ($oldentryp) {
+                $substitution = $self->_rewritePattern($substitution, '%-', ${$oldentryp});
+            }
 
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
-                $self->log(level => 'err', message => "delete rewrite \"${$dnp}\" failed");
+                $self->log(level => 'err', message => "delete rewrite rule \"$rule->{substitution}\" to \"${$dnp}\" failed");
+                return LDAP_OPERATIONS_ERROR;
+            }
+            (${$dnp}) = split(/\n/, $str);
+        }
+    }
+
+    return LDAP_SUCCESS;
+}
+
+sub post_delete
+{
+    my $self = shift;
+    my ($dnp, $argsp, $oldentryp) = @_;
+    my $conf = $self->{_config};
+
+    foreach my $rule (@{$conf->{rewrite}}) {
+        if ($rule->{context} eq 'deleteResult') {
+            if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
+                next;
+            }
+
+            my %rwcache;
+            my $substitution = $rule->{substitution};
+            $substitution = $self->_rewritePattern($substitution, '%0', ${$dnp});
+            if ($oldentryp) {
+                $substitution = $self->_rewritePattern($substitution, '%-', ${$oldentryp});
+            }
+
+            my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
+            if (!$str) {
+                $self->log(level => 'err', message => "delete result rewrite rule \"$rule->{substitution}\" to \"${$dnp}\" failed");
                 return LDAP_OPERATIONS_ERROR;
             }
             (${$dnp}) = split(/\n/, $str);
@@ -442,6 +607,9 @@ sub _checkConfig
             if ($type eq 'ldap') {
                 if (!defined($self->{ldapmap})) {$self->{ldapmap} = {}};
                 my $ldapmap = {};
+                if (Encode::is_utf8($conf->{rewritemap}{$map_name}->{attrs})) {
+                    $conf->{rewritemap}{$map_name}->{attrs} = encode('utf8', $conf->{rewritemap}{$map_name}->{attrs});
+                }
                 $self->_parseLdapUri($ldapmap, $conf->{rewritemap}{$map_name}->{attrs});
                 $self->{ldapmap}{$map_name} = $ldapmap;
             } elsif ($type eq 'lism') {
@@ -460,6 +628,13 @@ sub _checkConfig
                 if ($sizelimit) {
                     $lismmap->{sizelimit} = $sizelimit;
                 }
+                if (defined($conf->{rewritemap}{$map_name}->{match})) {
+                    $lismmap->{match} = $conf->{rewritemap}{$map_name}->{match};
+                }
+                if (defined($conf->{rewritemap}{$map_name}->{substitution})) {
+                    $lismmap->{substitution} = $conf->{rewritemap}{$map_name}->{substitution};
+                    $lismmap->{substitution} =~ s/%([0-9]+)/\$$1/;
+                }
                 $self->{lismmap}{$map_name} = $lismmap;
             }
         }
@@ -517,7 +692,7 @@ sub _rewritePattern
 sub _rewriteParse
 {
     my $self = shift;
-    my ($match, $substitution, $str, $rwcache) = @_;
+    my ($match, $substitution, $str, $rwcache, $is_dn) = @_;
     my $newstr;
 
     my @matches = ($str =~ /$match/gi);
@@ -531,6 +706,11 @@ sub _rewriteParse
         while ($substitution =~ /%$num/) {
             $substitution = $self->_rewritePattern($substitution, "%$num", $matches[$i]);
         }
+        my $escaped = $matches[$i];
+        $escaped =~ s/([\(\)*])/\\$1/g;
+        while ($substitution =~ /%\[${num}E\]/) {
+            $substitution = $self->_rewritePattern($substitution, '%\['.$num.'E\]', $escaped);
+        }
     }
 
     foreach my $substline ($self->_splitSubst($substitution)) {
@@ -551,7 +731,7 @@ sub _rewriteParse
                     return undef;
                 }
 
-                @values = $self->_rewriteMap($map_name, $map_args);
+                @values = $self->_rewriteMap($map_name, $map_args, $is_dn);
                 if (!defined($values[0])) {
                     return undef;
                 }
@@ -602,7 +782,7 @@ sub _splitSubst
     my $oldpos = 0;
     while ((my $pos = index($substitution, "\n", $oldpos)) > 0) {
         my $str = substr($substitution, $prevpos, $pos - $prevpos);
-        if (index($str, "%{", $oldpos - $prevpos) > 0) {
+        if (index($str, "%{", $oldpos - $prevpos) >= 0) {
             if (index($str, "(", $oldpos - $prevpos) > 0) {
                  $oldpos = $self->_passArgs($substitution, $oldpos);
             }
@@ -632,21 +812,38 @@ sub _passArgs
     my ($qtchar) = ($leftstr =~ /^ *(['"])/);
 
     while ($qtchar) {
+        $pos = index($str, $qtchar, $pos);
         while (1) {
-            $pos = index($str, $qtchar, $pos);
-            my $tmppos = index($str, $qtchar, $pos);
+            my $tmppos = index($str, $qtchar, $pos + 1);
             if ($tmppos < 0) {
                 last;
             }
-            $pos = $tmppos + 1;
+            $pos = $tmppos;
 
             if (substr($str, $tmppos - 1, 1) eq "\\") {
                 next;
             }
             last;
         }
+        $pos++;
+
         $leftstr = substr($str, $pos);
-        ($qtchar) = ($leftstr =~ / *(['"])/);
+        ($qtchar) = ($leftstr =~ /^ *, *(['"])/);
+        if (!$qtchar) {
+            my $tmppos = $pos;
+            while (1) {
+                $tmppos = index($str, ",", $tmppos + 1);
+                if ($tmppos < 0) {
+                    last;
+                }
+                $leftstr = substr($str, $tmppos);
+                ($qtchar) = ($leftstr =~ /^, *(['"])/);
+                if ($qtchar) {
+                    $pos = $tmppos;
+                    last;
+                }
+            }
+        }
     }
 
     $pos = index($str, ")", $pos);
@@ -660,13 +857,13 @@ sub _passArgs
 sub _rewriteMap
 {
     my $self = shift;
-    my ($map_name, $map_args) = @_;
+    my ($map_name, $map_args, $is_dn) = @_;
     my $conf = $self->{_config};
     my @values = ();
 
     if (defined($conf->{rewritemap}{$map_name})) {
         my $method = '_'.$conf->{rewritemap}{$map_name}->{type}.'Map';
-        @values = $self->$method($map_name, $map_args);
+        @values = $self->$method($map_name, $map_args, $is_dn);
     }
 
     return @values;
@@ -684,10 +881,33 @@ sub _ldapMap
 sub _lismMap
 {
     my $self = shift;
-    my ($map_name, $map_args) = @_;
+    my ($map_name, $map_args, $is_dn) = @_;
     my $lismmap = $self->{lismmap}{$map_name};
-
-    return $self->_searchLism($lismmap, $map_args);
+    my $escape = 0;
+    if ($map_args =~ /\\'/) {
+        $map_args =~ s/\\'/\\27/g;
+        $escape = 1;
+    }
+    my @args = ($map_args =~ /'([^']+)'/g);
+    if ($escape) {
+        for (my $i = 0; $i < @args; $i++) {
+            $args[$i] =~ s/\\27/\'/g;
+        }
+    }
+    my @vals = $self->_searchLism($lismmap, @args);
+    if ($is_dn && $lismmap->{attr} ne 'dn' && $lismmap->{attr} ne 'parentdn') {
+        for (my $i = 0; $i < @vals; $i++) {
+            $vals[$i] =~ s/\\/\\5C/g;
+            $vals[$i] =~ s/"/\\22/g;
+            $vals[$i] =~ s/#/\\23/g;
+            $vals[$i] =~ s/\+/\\2B/g;
+            $vals[$i] =~ s/\//\\2F/g;
+            $vals[$i] =~ s/;/\\3B/g;
+            $vals[$i] =~ s/</\\3C/g;
+            $vals[$i] =~ s/>/\\3E/g;
+        }
+    }
+    return @vals;
 }
 
 sub _functionMap
@@ -696,6 +916,8 @@ sub _functionMap
     my ($map_name, $map_args) = @_;
     my @values;
 
+    $map_args =~ s/(?<!\\)\\/\\\\/g;
+    $map_args =~ s/\\\\'(?!, *'| *\))/\\'/g;
     eval "\@values = $map_name($map_args)";
     if ($@) {
         $self->log(level => 'err', message => "rewriteMap $map_name failed: $@");
