@@ -2,7 +2,7 @@ package LISM::Handler::Script;
 
 use strict;
 use base qw(LISM::Handler);
-use Net::LDAP::Constant qw(:all);
+use LISM::Constant;
 use Data::Dumper;
 
 =head1 NAME
@@ -37,10 +37,10 @@ Do script before modify operation is done.
 sub pre_modify
 {
     my $self = shift;
-    my ($dnp, $listp) = @_;
+    my ($dnp, $listp, $errorp) = @_;
     my $conf = $self->{_config};
 
-    return $self->_do_modify('pre', $dnp, $listp);
+    return $self->_do_modify('pre', $dnp, $listp, $errorp);
 }
 
 =head2 post_modify($dnp, $listp)
@@ -52,7 +52,7 @@ Do script after modify operation is done.
 sub post_modify
 {
     my $self = shift;
-    my ($dnp, $listp, $oldentryp) = @_;
+    my ($dnp, $listp, $oldentryp, $errorp) = @_;
     my $conf = $self->{_config};
 
     return $self->_do_modify('post', $dnp, $listp, $oldentryp);
@@ -60,7 +60,7 @@ sub post_modify
 
 =pod
 
-=head2 pre_add($dnp, $entryStrp)
+=head2 pre_add($dnp, $entryStrp, $oldentryp, $errorp)
 
 Do script before add operation is done.
 
@@ -69,13 +69,13 @@ Do script before add operation is done.
 sub pre_add
 {
     my $self = shift;
-    my ($dnp, $entryStrp) = @_;
+    my ($dnp, $entryStrp, $oldentryp, $errorp) = @_;
     my $conf = $self->{_config};
 
-    return $self->_do_add('pre', $dnp, $entryStrp);
+    return $self->_do_add('pre', $dnp, $entryStrp, $oldentryp, $errorp);
 }
 
-=head2 post_add($dnp, $entryStrp)
+=head2 post_add($dnp, $entryStrp, $oldentryp, $errorp)
 
 Do script after add operation is done.
 
@@ -84,10 +84,10 @@ Do script after add operation is done.
 sub post_add
 {
     my $self = shift;
-    my ($dnp, $entryStrp) = @_;
+    my ($dnp, $entryStrp, $oldentryp, $errorp) = @_;
     my $conf = $self->{_config};
 
-    return $self->_do_add('post', $dnp, $entryStrp);
+    return $self->_do_add('post', $dnp, $entryStrp, $oldentryp, $errorp);
 }
 
 =pod
@@ -101,10 +101,10 @@ Do script before delete operation is done.
 sub pre_delete
 {
     my $self = shift;
-    my ($dnp) = @_;
+    my ($dnp, $errorp) = @_;
     my $conf = $self->{_config};
 
-    return $self->_do_delete('pre', $dnp);
+    return $self->_do_delete('pre', $dnp, undef, $errorp);
 }
 
 =head2 post_delete($dnp)
@@ -116,10 +116,10 @@ Do script after delete operation is done.
 sub post_delete
 {
     my $self = shift;
-    my ($dnp, $null, $oldentryp) = @_;
+    my ($dnp, $null, $oldentryp, $errorp) = @_;
     my $conf = $self->{_config};
 
-    return $self->_do_delete('post', $dnp, $oldentryp);
+    return $self->_do_delete('post', $dnp, $oldentryp, $errorp);
 }
 
 sub _checkConfig
@@ -154,12 +154,11 @@ Do script when modify operation is done.
 sub _do_modify
 {
     my $self = shift;
-    my ($type, $dnp, $listp, $oldentryp) = @_;
+    my ($type, $dnp, $listp, $oldentryp, $errorp) = @_;
     my $conf = $self->{_config};
     my $dn = ${$dnp};
     my $oldentry = defined($oldentryp) ? ${$oldentryp} : undef;
     my $rc = 0;
-    my $match = 0;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
 
@@ -173,6 +172,7 @@ sub _do_modify
             next;
         }
 
+        my $match = 0;
         my @info;
         my @list = @{$listp};
         while (@list > 0) {
@@ -223,7 +223,7 @@ sub _do_modify
         my $modinfo = "dn=$dn#".join('#', @info);
         foreach my $script (@{$rule->{op}{modify}->{script}}) {
             my $cmd = $script;
-            my $oldinfo = "dn=$dn#".$self->_parseEntry($oldentry);
+            my $oldinfo = "dn=$dn#".$self->_parseEntry($oldentry, $rule);
             my %params = ('%r' => $rdn_val, '%i' => $modinfo, '%o' => $oldinfo);
             $cmd = $self->_parseCommand($cmd, %params);
 
@@ -242,11 +242,12 @@ sub _do_modify
                 if ($rc) {
                     $self->log(level => 'err', message => "Script in modify failed($rc): $cmd");
                     $self->log(level => 'err', message => "Script messages: ".join(', ', @messages));
+                    ${$errorp} = join(', ', @messages);
                     if ($type eq 'post' && defined($rule->{op}{modify}->{rollback}) && $rule->{op}{modify}->{rollback} eq 'on') {
                         $self->log(level => 'err', message => "Rollback $dn in modify");
                         return LDAP_USER_CANCELED;
                     } else {
-                        return LDAP_OPERATIONS_ERROR;
+                        return ($rc == 32 || $rc == 68) ? $rc : LDAP_OPERATIONS_ERROR;
                     }
                 } else {
                     $self->log(level => 'info', message => "Script in modify succeeded: $cmd");
@@ -262,7 +263,7 @@ sub _do_modify
 
 =pod
 
-=head2 _do_add($type, $dnp, $entryStrp)
+=head2 _do_add($type, $dnp, $entryStrp, $oldentryp, $errorp)
 
 Do script when add opeartion is done.
 
@@ -271,7 +272,7 @@ Do script when add opeartion is done.
 sub _do_add
 {
     my $self = shift;
-    my ($type, $dnp,  $entryStrp) = @_;
+    my ($type, $dnp,  $entryStrp, $oldentryp, $errorp) = @_;
     my $conf = $self->{_config};
     my $dn = ${$dnp};
     my $entryStr = ${$entryStrp}[0];
@@ -316,11 +317,12 @@ sub _do_add
                 if ($rc) {
                     $self->log(level => 'err', message => "Script in add failed($rc): $cmd");
                     $self->log(level => 'err', message => "Script messages: ".join(', ', @messages));
+                    ${$errorp} = join(', ', @messages);
                     if ($type eq 'post' && defined($rule->{op}{add}->{rollback}) && $rule->{op}{add}->{rollback} eq 'on') {
                         $self->log(level => 'err', message => "Rollback $dn in add");
                         return LDAP_USER_CANCELED;
                     } else {
-                        return LDAP_OPERATIONS_ERROR;
+                        return ($rc == 32 || $rc == 68) ? $rc : LDAP_OPERATIONS_ERROR;
                     }
                 } else {
                     $self->log(level => 'info', message => "Script in add succeeded: $cmd");
@@ -336,7 +338,7 @@ sub _do_add
 
 =pod
 
-=head2 _do_delete($type, $dnp)
+=head2 _do_delete($type, $dnp, $errorp)
 
 Do script when delete operation is done.
 
@@ -345,7 +347,7 @@ Do script when delete operation is done.
 sub _do_delete
 {
     my $self = shift;
-    my ($type, $dnp, $oldentryp) = @_;
+    my ($type, $dnp, $oldentryp, $errorp) = @_;
     my $conf = $self->{_config};
     my $dn = ${$dnp};
     my $oldentry = defined($oldentryp) ? ${$oldentryp} : undef;
@@ -370,7 +372,7 @@ sub _do_delete
 
             foreach my $script (@{$rule->{op}{$op}->{script}}) {
                 my $cmd = $script;
-                my $oldinfo = "dn=$dn#".$self->_parseEntry($oldentry);
+                my $oldinfo = "dn=$dn#".$self->_parseEntry($oldentry, $rule);
                 my %params = ('%r' => $rdn_val, '%i' => $info, '%o' => $oldinfo);
                 $cmd = $self->_parseCommand($cmd, %params);
 
@@ -388,11 +390,12 @@ sub _do_delete
                     if ($rc) {
                         $self->log(level => 'err', message => "Script in delete failed($rc): $cmd");
                         $self->log(level => 'err', message => "Script messages: ".join(', ', @messages));
+                        ${$errorp} = join(', ', @messages);
                         if ($type eq 'post' && defined($rule->{op}{delete}->{rollback}) && $rule->{op}{delete}->{rollback} eq 'on') {
                             $self->log(level => 'err', message => "Rollback $dn in delete");
                             return LDAP_USER_CANCELED;
                         } else {
-                            return LDAP_OPERATIONS_ERROR;
+                            return ($rc == 32 || $rc == 68) ? $rc : LDAP_OPERATIONS_ERROR;
                         }
                     } else {
                         $self->log(level => 'info', message => "Script in delete succeeded: $cmd");
