@@ -1,15 +1,18 @@
 package LISM::Storage;
 
 use strict;
-use Net::LDAP::Constant qw(:all);
+use LISM::Constant;
 use Digest::MD5;
 use Digest::SHA1;
 use MIME::Base64;
 use POSIX;
 use Encode;
+use Scalar::Util qw(weaken);
 use Data::Dumper;
 if ($^O ne 'MSWin32') {
     eval "use Sys::Syslog";
+} else {
+    eval "use Log::Dispatch::FileRotate";
 }
 
 our $controlAttr = 'lismControl';
@@ -32,13 +35,15 @@ This is a plain constructor.
 sub new
 {
     my $class = shift;
-    my ($suffix, $contentry) = @_;
+    my ($suffix, $contentry, $lism) = @_;
 
     my $this = {};
     bless $this, $class;
 
     $this->{suffix} = $suffix;
     $this->{contentrystr} = $contentry;
+    $this->{lism} = $lism;
+    weaken($this->{lism});
 
     return $this;
 }
@@ -174,6 +179,17 @@ sub search
     my $conf = $self->{_config};
     my @match_entries = ();
 
+    $base =~ s/\\22/"/gi;
+    $base =~ s/\\23/#/gi;
+    $base =~ s/\\2B/+/gi;
+    $base =~ s/\\2F/\//gi;
+    $base =~ s/\\3B/;/gi;
+    $base =~ s/\\3C/</gi;
+    $base =~ s/\\3E/>/gi;
+    $base =~ s/\\3D/=/gi;
+    $base =~ s/\\5C/\\/gi;
+    $base =~ s/\\(?=[+#"])//gi;
+
     if ($filterStr =~ /^\(&\($controlAttr=[^\)]+\)/i) {
         $filterStr =~ s/^\(&\($controlAttr=[^\)]+\)//;
         $filterStr =~ s/\)$//;
@@ -245,7 +261,7 @@ sub search
             }
         } elsif ($objscope !=1 && $self->parseFilter($filter, $entry)) {
             push(@match_entries, $entry);
-            $sizeLim = $sizeLim - $entry;
+            $sizeLim = $sizeLim - 1;
         }
 
         # search entries below base
@@ -341,6 +357,16 @@ sub modify
     my $self = shift;
     my ($dn, @list) = @_;
     my $conf = $self->{_config};
+    $dn =~ s/\\22/"/gi;
+    $dn =~ s/\\23/#/gi;
+    $dn =~ s/\\2B/+/gi;
+    $dn =~ s/\\2F/\//gi;
+    $dn =~ s/\\3B/;/gi;
+    $dn =~ s/\\3C/</gi;
+    $dn =~ s/\\3E/>/gi;
+    $dn =~ s/\\3D/=/gi;
+    $dn =~ s/\\5C/\\/gi;
+    $dn =~ s/\\(?=[+#"])//gi;
 
     my $retry = 0;
     while (1) {
@@ -377,9 +403,20 @@ sub add
     my $self = shift;
     my ($dn, $entryStr) = @_;
     my $conf = $self->{_config};
+    $dn =~ s/\\22/"/gi;
+    $dn =~ s/\\23/#/gi;
+    $dn =~ s/\\2B/+/gi;
+    $dn =~ s/\\2F/\//gi;
+    $dn =~ s/\\3B/;/gi;
+    $dn =~ s/\\3C/</gi;
+    $dn =~ s/\\3E/>/gi;
+    $dn =~ s/\\3D/=/gi;
+    $dn =~ s/\\5C/\\/gi;
+    $dn =~ s/\\(?=[+#"])//gi;
 
     # check rdn's value
     my ($rdn, $rdn_val) = ($dn =~ /^([^=]+)=([^,]+),/);
+    $rdn_val = $self->_unescapedn($rdn_val);
     $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
     if ($entryStr !~ /^$rdn: $rdn_val *$/mi) {
         return LDAP_NAMING_VIOLATION;
@@ -465,6 +502,16 @@ sub delete
     my $self = shift;
     my ($dn) = @_;
     my $conf = $self->{_config};
+    $dn =~ s/\\22/"/gi;
+    $dn =~ s/\\23/#/gi;
+    $dn =~ s/\\2B/+/gi;
+    $dn =~ s/\\2F/\//gi;
+    $dn =~ s/\\3B/;/gi;
+    $dn =~ s/\\3C/</gi;
+    $dn =~ s/\\3E/>/gi;
+    $dn =~ s/\\3D/=/gi;
+    $dn =~ s/\\5C/\\/gi;
+    $dn =~ s/\\(?=[+#"])//gi;
 
     my $retry = 0;
     while (1) {
@@ -580,11 +627,13 @@ sub log
 
     if ($^O ne 'MSWin32') {
         openlog('LISM', 'pid', $self->{lism}->{_config}->{syslogfacility});
-        setlogmask(Sys::Syslog::LOG_UPTO(Sys::Syslog::xlate($conf->{sysloglevel})));
+        if ($conf->{sysloglevel} ne 'info') {
+            setlogmask(Sys::Syslog::LOG_UPTO(Sys::Syslog::xlate($conf->{sysloglevel})));
+        }
         syslog($p{'level'}, $p{'message'});
         closelog();
     } else {
-        print STDERR strftime("%Y %m %d %H:%M:%S", localtime(time))." $p{'message'}\n";
+        $self->{log}->log(level => $p{'level'}, message => strftime("%Y/%m/%d %H:%M:%S", localtime(time))." ".$p{'message'}."\n");
     }
 
     if (defined($conf->{printlog}) && $conf->{printlog} =~ /$p{'level'}/) {
@@ -616,7 +665,11 @@ sub hashPasswd
     if ($pwhtype) {
         # already hashed password
         if ($pwhtype ne $htype) {
-            return undef;
+            if ($htype eq 'PLAINTEXT') {
+                return $passwd;
+            } else {
+                return undef;
+            }
         }
 
         $passwd =~ s/^\{[^\}]+\}//;
@@ -627,6 +680,10 @@ sub hashPasswd
         return $passwd;
     }
 
+    if ($htype =~ /^(CRYPT|MD5|SHA)$/i && Encode::is_utf8($passwd)) {
+        $passwd = encode('utf8', $passwd);
+    }
+
     # hash the password
     for (my $i = 0; $i < $num; $i++) {
         if ($htype =~ /^CRYPT$/i) {
@@ -789,7 +846,9 @@ sub buildEntryStr
 
 sub manageDIT
 {
-    return 0;
+    my $self = shift;
+
+    return $self->{_manageDIT};
 }
 
 
@@ -808,6 +867,32 @@ sub _checkConfig
         }
     }
 
+    if ($^O eq 'MSWin32') {
+        if (!defined($conf->{logfile})) {
+            return 1;
+        }
+
+        my $timezone = 'JST';
+        my $rotatedate = 'yyyy-MM-dd';
+        my $rotatenum = 4;
+        if (defined($conf->{logtimezone})) {
+            $timezone = $conf->{timezone};
+        }
+        if (defined($conf->{logrotatedate})) {
+            $rotatedate = $conf->{logrotatedate};
+        }
+        if (defined($conf->{logrotatenum})) {
+            $rotatenum = $conf->{logrotatenum};
+        }
+        $self->{log} = Log::Dispatch::FileRotate->new(name => 'LISM',
+                                                     min_level => $conf->{sysloglevel},
+                                                     filename => $conf->{logfile},
+                                                     mode => 'append',
+                                                     TZ => $timezone,
+                                                     DatePattern => $rotatedate,
+                                                     max => $rotatenum);
+    }
+
     # hash type
     if (!defined($conf->{hash})) {
         $conf->{hash} = 'PLAINTEXT';
@@ -839,11 +924,17 @@ sub _checkConfig
     # connection
     if (!defined($conf->{connection}[0]->{retry})
         || $conf->{connection}[0]->{retry}[0] !~ /^[0-9]+$/) {
-        $conf->{connection}[0]->{retry}[0] = 0;
+        $conf->{connection}[0]->{retry}[0] = 3;
     }
     if (!defined($conf->{connection}[0]->{interval})
         || $conf->{connection}[0]->{interval}[0] !~ /^[0-9]+$/) {
-        $conf->{connection}[0]->{interval}[0] = 10;
+        $conf->{connection}[0]->{interval}[0] = 0;
+    }
+
+    if (defined($conf->{manageDIT}) && $conf->{manageDIT}[0]) {
+        $self->{_manageDIT} = 1;
+    } else {
+        $self->{_manageDIT} = 0;
     }
 
     # data object
@@ -939,6 +1030,8 @@ sub _checkConfig
             if (!defined($oconf->{rdn}) || !defined($oconf->{attr}{$oconf->{rdn}[0]})) {
                 $self->log(level => 'alert', message => "Rdn of $oname object is invalid");
                 return 1;
+            } else {
+                $oconf->{rdn}[0] = encode('utf8', $oconf->{rdn}[0]);
             }
 
             if (defined($oconf->{strginfo})) {
@@ -1062,6 +1155,7 @@ sub _getObject
     my @objs = ($obj);
     my @pkeyarr = ();
     my $key;
+    my $container_oname;
     for (my $base = $obj->{suffix};1; $base = pop(@rdn_list).','.$base) {
         my $rc;
         my $entry;
@@ -1070,6 +1164,7 @@ sub _getObject
             if (defined($objs[$i]->{suffix}) && !$objs[$i]->{entrystr}) {
                 $entry = "top_container";
                 $obj = $objs[$i];
+                $container_oname = undef;
                 last;
             }
             if (defined($objs[$i]->{conf}->{subcontainer})) {
@@ -1077,6 +1172,7 @@ sub _getObject
                 if ($base =~ /^$rdn,/i) {
                     $key = undef;
                     $entry = "subcontainer";
+                    $container_oname = $objs[$i]->{name};
                     last;
                 }
             }
@@ -1084,6 +1180,7 @@ sub _getObject
             ($rc, $key, $entry) = $self->_baseSearch($objs[$i], \@pkeyarr, $base, 0, 0, 1, 0, undef);
             if ($entry) {
                 $obj = $objs[$i];
+                $container_oname = undef;
                 last;
             }
         }
@@ -1107,6 +1204,10 @@ sub _getObject
 
     if (defined($obj->{child})) {
         foreach my $child (@{$obj->{child}}) {
+            if ($container_oname && $container_oname ne $child->{name}) {
+                next;
+            }
+
             if (defined($child->{conf}->{subcontainer}) &&
                 ($dn =~ /^$child->{conf}->{subcontainer}[0]->{rdn}[0],/i ||
                     $dn =~ /^$child->{conf}->{rdn}[0]=[^,]+,$child->{conf}->{subcontainer}[0]->{rdn}[0]/i)) {
@@ -1289,6 +1390,15 @@ sub _childSearch
             if (!$childdn) {
                 return (LDAP_OTHER, \@match_keys, @match_entries);
             }
+            $childdn =~ s/\\22/"/gi;
+            $childdn =~ s/\\23/#/gi;
+            $childdn =~ s/\\2B/+/gi;
+            $childdn =~ s/\\2F/\//gi;
+            $childdn =~ s/\\3B/;/gi;
+            $childdn =~ s/\\3C/</gi;
+            $childdn =~ s/\\3E/>/gi;
+            $childdn =~ s/\\3D/=/gi;
+            $childdn =~ s/\\5C/\\/gi;
 
             ($rc, $keys, @entries) = $self->_childSearch($objs[$i], [@pkeyarr, ${$ckeys}[$j]], $childdn, $scope, $deref, $sizeLim, $timeLim, $filter, $attrOnly, @attrs);
             if ($rc) {
@@ -1640,6 +1750,25 @@ sub random
     return $string;
 }
 
+sub _unescapedn
+{
+    my $self = shift;
+    my ($dn) = @_;
+
+    $dn =~ s/\\(?:3C|<)/</gi;
+    $dn =~ s/\\(?:3E|>)/>/gi;
+    $dn =~ s/\\(?:2C|,)/,/gi;
+    $dn =~ s/\\22/"/gi;
+    $dn =~ s/\\23/#/gi;
+    $dn =~ s/\\2B/+/gi;
+    $dn =~ s/\\2F/\//gi;
+    $dn =~ s/\\3B/;/gi;
+    $dn =~ s/\\3D/=/gi;
+    $dn =~ s/\\5C/\\/gi;
+
+    return $dn;
+}
+
 =head1 SEE ALSO
 
 L<LISM>
