@@ -6,7 +6,7 @@ use Net::LDAP;
 use Net::LDAP::Control::Paged;
 use Net::LDAP::Control::VLV;
 use Net::LDAP::Control::Sort;
-use Net::LDAP::Constant qw(:all);
+use LISM::Constant;
 use MIME::Base64;
 use Encode;
 use Data::Dumper;
@@ -293,6 +293,10 @@ sub modify
             push(@values, shift @list);
         }
 
+        if (defined($conf->{operationalattr}) && grep(/^$key$/i, @{$conf->{operationalattr}})) {
+            next;
+        }
+
         if ($key =~ /^(modifyTimestamp|plainPassword)$/i) {
             next;
         }
@@ -373,6 +377,16 @@ sub add
     my ($dn,  $entryStr) = @_;
     my $conf = $self->{_config};
     my $rc = LDAP_SUCCESS;
+    my $orgdn = $dn;
+    $dn =~ s/\\22/\\"/gi;
+    $dn =~ s/\\23/\\#/gi;
+    $dn =~ s/\\2B/\\+/gi;
+    $dn =~ s/\\2F/\//gi;
+    $dn =~ s/\\3B/\\;/gi;
+    $dn =~ s/\\3C/\\</gi;
+    $dn =~ s/\\3E/\\>/gi;
+    $dn =~ s/\\3D/=/gi;
+    $dn =~ s/\\5C/\\\\/gi;
 
     if (defined($conf->{noop}) && grep(/^add$/i, @{$conf->{noop}})) {
         return $rc;
@@ -411,7 +425,11 @@ sub add
             ($key, $val) = split(/:: /, $attr);
             $val = decode_base64($val);
         } else {
-            ($key, $val) = split(/: /, $attr);
+            ($key, $val) = split(/: ?/, $attr, 2);
+        }
+
+        if (defined($conf->{operationalattr}) && grep(/^$key$/i, @{$conf->{operationalattr}})) {
+            next;
         }
 
         if ($key =~ /^(createTimestamp|modifyTimestamp|plainpassword)$/i) {
@@ -450,15 +468,36 @@ sub add
             if (!defined($deleteflag->{ovrfilter})) {
                 next;
             }
+            if (defined($deleteflag->{dn}) && $dn !~ /$deleteflag->{dn}/i) {
+                next;
+            }
 
-            my $delentry;
-            ($rc, $delentry) = $self->_do_search(undef, $dn, 0, 0, 1, 0, $deleteflag->{ovrfilter}, 0, 'objectClass');
+            my ($parentdn) = ($orgdn =~ /^[^,]+,(.+)$/);
+            my $parentEntry;
+            ($rc, $parentEntry) = $self->_do_search(undef, $parentdn, 0, 0, 1, 0, "(!($key=$deleteflag->{value}))", 0, 'objectClass');
+            if ($rc) {
+                last;
+            } elsif (!$parentEntry) {
+                return LDAP_NO_SUCH_OBJECT;
+            }
+
+            my @delentries;
+            ($rc, @delentries) = $self->_do_search(undef, $dn, 2, 0, 0, 0, $deleteflag->{ovrfilter}, 0, 'objectClass');
             if ($rc) {
                 last;
             }
-            if ($delentry) {
-                my $delmsg = $self->{ldap}->delete($dn);
-                if (!$delmsg->code) {
+            if (@delentries) {
+                @delentries = sort {length $b <=> length $a} @delentries;
+                foreach my $delentry (@delentries) {
+                    my ($deldn) = ($delentry =~ /^dn: ([^\n]+)/);
+                    $deldn =~ s/$self->{suffix}/$conf->{nc}/i;
+                    my $delmsg = $self->{ldap}->delete($deldn);
+                    if ($delmsg->code) {
+                        $rc = $delmsg->code;
+                        last;
+                    }
+                }
+                if (!$rc) {
                     $msg = $self->{ldap}->add($dn, attrs => [%attrs]);
                 }
                 last;
@@ -530,7 +569,7 @@ sub delete
 
     if (defined($conf->{deleteflag})) {
         my $entryStr;
-        ($rc, $entryStr) = $self->_do_search(undef, $dn, 0, 0, 1, 0, '(objectClass=*)');
+        ($rc, $entryStr) = $self->_do_search(undef, $dn, 0, 0, 1, 0, '(objectClass=*)', 0, 'objectClass');
         if ($rc) {
             return $rc;
         }
@@ -538,6 +577,16 @@ sub delete
         my $match = 0;
         foreach my $key (keys %{$conf->{deleteflag}}) {
             my $deleteflag = $conf->{deleteflag}{$key};
+            if (defined($deleteflag->{dn}) && $dn !~ /$deleteflag->{dn}/i) {
+                next;
+            }
+            my @entries;
+            ($rc, @entries) = $self->_do_search(undef, $dn, 1, 0, 0, 0, "(!($key=$deleteflag->{value}))", 0, 'objectClass');
+            if ($rc) {
+                return $rc;
+            } elsif (@entries) {
+                return LDAP_NOT_ALLOWED_ON_NONLEAF
+            }
             if (!defined($deleteflag->{filter}) || $self->parseFilter($deleteflag->{filterobj}, $entryStr)) {
                 $rc = $self->modify($dn, "REPLACE", $key, $deleteflag->{value});
                 $match = 1;
@@ -641,6 +690,9 @@ sub hashPasswd
     my $hashpw;
 
     my ($htype, $otype) = split(/:/, $conf->{hash});
+    if ($passwd =~ /^{([^}]+)}/ && $htype ne $1) {
+        return $passwd;
+    }
 
     my $hashpw = $self->SUPER::hashPasswd($passwd, $salt);
 
@@ -669,7 +721,7 @@ sub _getConnect
         (!defined($conf->{connection}[0]->{type}) || $conf->{connection}[0]->{type}[0] ne 'every')) {
         my $msg = $self->{ldap}->bind($conf->{binddn}[0], password => $conf->{bindpw}[0]);
         if ($msg->code) {
-            $self->log(level => 'err', message => "Connection check failed");
+            $self->log(level => 'err', message => "Connection check($conf->{uri}[0]) failed: ".$msg->error);
             $self->{ldap}->unbind();
             $self->{bind}->unbind();
             undef($self->{ldap});
@@ -683,7 +735,7 @@ sub _getConnect
     $self->{bind} = Net::LDAP->new($conf->{uri}[0], timeout => $TIMEOUT);
 
     if (!defined($self->{ldap}) || !defined($self->{bind})) {
-        $self->log(level => 'alert', message => "Can't connect $conf->{uri}[0]");
+        $self->log(level => 'alert', message => "Can't connect $conf->{uri}[0]".($@ ? ": $@" : ''));
         return -1;
     }
 
@@ -750,6 +802,13 @@ sub _checkConfig
         }
     }
 
+    if (defined($conf->{defaultattrs})) {
+        my @attrs = split(/, */, $conf->{defaultattrs}[0]);
+        if (@attrs) {
+            $conf->{defattrs} = \@attrs;
+        }
+    }
+
     return $rc;
 }
 
@@ -860,6 +919,10 @@ sub _do_search
         return LDAP_SERVER_DOWN;
     }
 
+    if (!@attrs && defined($conf->{defattrs})) {
+        @attrs = @{$conf->{defattrs}};
+    }
+
     $filterStr = decode('utf8', $filterStr);
 
     # Attribute mapping
@@ -873,6 +936,9 @@ sub _do_search
         } elsif ($ldapmap->{type} =~ /^attribute$/i) {
             $base =~ s/^$ldapmap->{local}=/$ldapmap->{foreign}=/i;
             $filterStr =~ s/\($ldapmap->{local}=/\($ldapmap->{foreign}=/gmi;
+            for (my $i = 0; $i < @attrs; $i++) {
+                $attrs[$i] =~ s/^$ldapmap->{local}$/$ldapmap->{foreign}/i;
+            }
         }
     }
 
@@ -904,11 +970,50 @@ sub _do_search
                 my $dn = decode('utf8', $entry->dn);
 
                 $dn =~ s/$conf->{nc}$/$self->{suffix}/i;
+                $dn =~ s/\\"/\\22/g;
+                $dn =~ s/\\#/\\23/g;
+                $dn =~ s/\\\+/\\2B/g;
+                $dn =~ s/\\\//\\2F/g;
+                $dn =~ s/\\;/\\3B/g;
+                $dn =~ s/\\</\\3C/g;
+                $dn =~ s/\\>/\\3E/g;
+                $dn =~ s/\\=/\\3D/g;
+                $dn =~ s/\\\\/\\5C/g;
                 my $entryStr = "dn: $dn\n";
                 if ($dn =~ /^$self->{suffix}$/i) {
                     next;
                 } else {
                     foreach my $attr ($entry->attributes) {
+                        if ($attr =~ /^member;range=[0-9]+-([0-9]+)$/i) {
+                            my $start = $1 + 1;
+                            my $end = $1 + 1500;
+                            my @values = $entry->get_value($attr);
+                            for (my $k = 0; $k < 100; $k++) {
+                                my $gmsg = $self->{ldap}->search(base => $entry->dn, scope => 0, deref => $deref, filter => '(objectClass=*)', attrs => ["member;range=${start}-${end}"]);
+                                if ($gmsg->code) {
+                                    $self->_freeConnect($msg, defined($ctrl_class{'Net::LDAP::Control::VLV'}));
+                                    return ($gmsg->code, @match_entries);
+                                }
+                                my $gentry = $gmsg->entry(0);
+                                if ($gentry->exists("member;range=${start}-*")) {
+                                    unshift(@values, $gentry->get_value("member;range=${start}-*"));
+                                    last;
+                                } else {
+                                    unshift(@values, $gentry->get_value("member;range=${start}-${end}"));
+                                }
+                                if ($gmsg->count < 1500) {
+                                    last;
+                                }
+                                $start += 1500;
+                                $end += 1500;
+                            }
+                            foreach my $value (@values) {
+                                $value = decode('utf8', $value);
+                                $value =~ s/$conf->{nc}$/$self->{suffix}/i;
+                                $entryStr = $entryStr."member: $value\n";
+                            }
+                            next;
+                        }
                         foreach my $value ($entry->get_value($attr)) {
                             if ($attr =~ /$rawattrs/i) {
                                 $value = encode_base64($value, '');
