@@ -4,7 +4,7 @@ use strict;
 use URI;
 use Net::LDAP;
 use Net::LDAP::Util qw(ldap_error_text);
-use Net::LDAP::Constant qw(:all);
+use LISM::Constant;
 
 our $RETRY = 3;
 
@@ -81,7 +81,7 @@ sub searchLdap
                     ($key) = ($entryStr =~ /^$param->{id}->{value}:\:? (.*)$/mi);
                 }
             } else {
-                ($key) = ($entry =~ /^dn: [^=]+=([^,]+),/i);
+                ($key) = ($entryStr =~ /^dn: [^=]+=([^,]+),/i);
             }
 
             push(@keys, $key);
