@@ -13,6 +13,7 @@ use Time::Local;
 use Time::HiRes qw(gettimeofday);
 use POSIX;
 use Encode;
+use MIME::Base64;
 use Data::Dumper;
 
 sub date2time
@@ -43,12 +44,23 @@ sub time2date
 
 sub getValue
 {
-    my ($entryStr, $attr, $default) = @_;
+    my ($entryStr, $attr, $default, $escape) = @_;
 
-    my $value = ($entryStr =~ /^$attr:\:? (.*)$/mi)[0];
-    if (!defined($value) && $default) {
+    my $value = ($entryStr =~ /^$attr:\:? +(.*)$/mi)[0];
+    if (!defined($value) && defined($default)) {
         $value = $default;
     }
+    if ($value && $escape) {
+        $value =~ s/"/\\22/g;
+        $value =~ s/#/\\23/g;
+        $value =~ s/\+/\\2B/g;
+        $value =~ s/,/\\2C/g;
+        $value =~ s/\//\\2F/g;
+        $value =~ s/;/\\3B/g;
+        $value =~ s/</\\3C/g;
+        $value =~ s/>/\\3E/g;
+        $value =~ s/=/\\3D/g;
+    }
 
     return $value;
 }
@@ -159,4 +171,79 @@ sub regmatch
     return @vals;
 }
 
+sub dn2oupath
+{
+    my ($dn, $base, $normalize) = @_;
+    if ($dn !~ /,$base.*$/i) {
+        return '';
+    }
+    $dn =~ s/,$base.*$//i;
+    my @orgs = ($dn =~ /ou=([^,]+)/gi);
+    if (@orgs) {
+        my $oupath = join('/', reverse(@orgs));
+        if ($normalize) {
+            $oupath =~ tr/A-Z/a-z/;
+        }
+        return $oupath;
+    } else {
+        return '';
+    }
+}
+
+sub getParent
+{
+    my ($entryStr) = @_;
+
+    my ($parent) = ($entryStr =~ /^[^,]+,[^=]+=([^,]+),/);
+    return $parent;
+}
+
+sub path2dn
+{
+    my ($path, $attr, $isReverse) = @_;
+    $path =~ s/[^=]+=//;
+
+    my $dn = '';
+    my @matches = split(/(?<!\\)\//, $path);
+    if ($isReverse == '1') {
+        @matches = reverse(@matches);
+    }
+    for (my $i = 0; $i < @matches; $i++) {
+        $matches[$i] =~ s/\\\//\//g;
+        $matches[$i] =~ s/\\/\\5C/g;
+        $matches[$i] =~ s/"/\\22/g;
+        $matches[$i] =~ s/#/\\23/g;
+        $matches[$i] =~ s/\+/\\2B/g;
+        $matches[$i] =~ s/;/\\3B/g;
+        $matches[$i] =~ s/</\\3C/g;
+        $matches[$i] =~ s/>/\\3E/g;
+        $matches[$i] =~ s/=/\\3D/g;
+        $dn .= $attr.'='.$matches[$i];
+        if ($i != scalar(@matches)-1) {
+        	$dn .= ',';
+        }
+    }
+    return $dn;
+}
+
+sub regexCount
+{
+    my ($str, $regex) = @_;
+    my $num = 0;
+
+    if ($str =~ /$regex/i) {
+        my @vals = ($str =~ /$regex/gmi);
+        $num = @vals;
+    }
+
+    return $num;
+}
+
+sub binToHex
+{
+    my ($str) = @_;
+
+    return unpack("H*", decode_base64($str));
+}
+
 1;
