@@ -7,6 +7,9 @@ fi
 INFO=$3
 INFO=`echo $INFO | sed s/#[^#]*createtimestamp=[0-9]*Z//`
 INFO=`echo $INFO | sed s/#[^#]*modifytimestamp=[0-9]*Z//`
-echo "rdn=$2#$INFO" | sed s/#/,/g > $1
+OINFO=$4
+OINFO=`echo $OINFO | sed s/#[^#]*createtimestamp=[0-9]*Z//`
+OINFO=`echo $OINFO | sed s/#[^#]*modifytimestamp=[0-9]*Z//`
+echo "rdn=$2#$INFO;$OINFO" | sed s/#/,/g > $1
 
 exit 0
